package models

import (
	"eta/eta_index_lib/global"
	"eta/eta_index_lib/utils"
	"fmt"
	"reflect"
	"strconv"
	"strings"
	"time"
)

//路透

// 新增路透指标数据
func AddEdbDataFromLt(edbCode string, ltDataList map[int64]interface{}) (err error) {
	//o := orm.NewOrm()
	if len(ltDataList) > 0 {
		var isAdd bool
		addSql := ` INSERT INTO edb_data_lt(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
		addSql = utils.ReplaceDriverKeywords("", addSql)
		for timestampInt, edbValue := range ltDataList {
			dataTime := time.Unix(timestampInt/1000, 0)

			//校验数据类型对不对
			valType := reflect.TypeOf(edbValue)
			if valType == nil {
				continue
			}
			if valType.String() != "float64" {
				continue
			}
			sValue := edbValue.(float64)

			eDate := dataTime.Format(utils.FormatDate)
			if err != nil {
				return err
			}
			timestamp := dataTime.UnixNano() / 1e6
			timeStr := fmt.Sprintf("%d", timestamp)
			addSql += GetAddSql("0", edbCode, eDate, timeStr, utils.SubFloatToString(sValue, 20))
			isAdd = true
		}
		if isAdd {
			addSql = strings.TrimRight(addSql, ",")
			//_, err = o.Raw(addSql).Exec()
			err = global.DEFAULT_DB.Exec(addSql).Error
			if err != nil {
				fmt.Println("RefreshAllEdbDataByPb add Err", err.Error())
				return
			}
		}
	}
	return
}

// 刷新路透指标数据
func RefreshEdbDataFromLt(edbInfoId int, edbCode, startDate string, ltDataList map[int64]interface{}) (err error) {
	source := utils.DATA_SOURCE_LT
	subSource := utils.DATA_SUB_SOURCE_EDB

	//o := orm.NewOrm()

	// 真实数据的最大日期  ， 插入规则配置的日期
	var realDataMaxDate, edbDataInsertConfigDate time.Time
	var edbDataInsertConfig *EdbDataInsertConfig
	var isFindConfigDateRealData bool //是否找到配置日期的实际数据的值
	{
		edbDataInsertConfig, err = GetEdbDataInsertConfigByEdbId(edbInfoId)
		if err != nil && !utils.IsErrNoRow(err) {
			return
		}
		if edbDataInsertConfig != nil {
			edbDataInsertConfigDate = edbDataInsertConfig.Date
		}
	}

	var existCondition string
	var existPars []interface{}

	existCondition += " AND edb_info_id=? "
	existPars = append(existPars, edbInfoId)
	if startDate != "" {
		existCondition += " AND data_time>=? "
		existPars = append(existPars, startDate)
	}

	existList, err := GetEdbDataByCondition(source, subSource, existCondition, existPars)
	if err != nil {
		return err
	}
	existMap := make(map[string]*EdbInfoSearchData)
	for _, v := range existList {
		existMap[v.DataTime] = v
	}

	edbInfoIdStr := strconv.Itoa(edbInfoId)

	addSql := ` INSERT INTO edb_data_lt(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	addSql = utils.ReplaceDriverKeywords("", addSql)
	var isAdd bool

	for timestampInt, edbValue := range ltDataList {
		dataTime := time.Unix(timestampInt/1000, 0)

		//校验数据类型对不对
		valType := reflect.TypeOf(edbValue)
		if valType == nil {
			continue
		}
		if valType.String() != "float64" {
			continue
		}
		sValue := edbValue.(float64)

		eDate := dataTime.Format(utils.FormatDate)
		if err != nil {
			return err
		}
		dataTime, _ = time.ParseInLocation(utils.FormatDate, eDate, time.Local) //这么弄的主要目的是为了北京时间

		saveValue := utils.SubFloatToString(sValue, 30)
		if findItem, ok := existMap[eDate]; !ok {
			timestamp := dataTime.UnixNano() / 1e6
			timeStr := fmt.Sprintf("%d", timestamp)

			addSql += GetAddSql(edbInfoIdStr, edbCode, eDate, timeStr, saveValue)
			isAdd = true
		} else {
			if findItem != nil && utils.SubFloatToString(findItem.Value, 30) != saveValue {
				err = ModifyEdbDataById(source, subSource, findItem.EdbDataId, saveValue)
				if err != nil {
					return err
				}
			}
		}

		// 下面代码主要目的是处理掉手动插入的数据判断
		{
			if realDataMaxDate.IsZero() || dataTime.After(realDataMaxDate) {
				realDataMaxDate = dataTime
			}
			if edbDataInsertConfigDate.IsZero() || dataTime.Equal(edbDataInsertConfigDate) {
				isFindConfigDateRealData = true
			}
		}
	}

	// 处理手工数据补充的配置
	HandleConfigInsertEdbData(realDataMaxDate, edbDataInsertConfig, edbInfoId, source, subSource, existMap, isFindConfigDateRealData)

	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		//_, err = o.Raw(addSql).Exec()
		err = global.DEFAULT_DB.Exec(addSql).Error
		if err != nil {
			fmt.Println("RefreshAllEdbDataByLt add Err", err.Error())
			return
		}
	}
	return
}
