package models

import (
	"eta/eta_index_lib/global"
	"eta/eta_index_lib/utils"
	"fmt"
	"gorm.io/gorm"
	"strconv"
	"strings"
	"time"
)

// BaseFromNationalStatisticsData 国家统计局-指标数据表
type BaseFromNationalStatisticsData struct {
	BaseFromNationalStatisticsDataId int `gorm:"column:base_from_national_statistics_data_id;primaryKey"`
	//BaseFromNationalStatisticsDataId  int `orm:"column(base_from_national_statistics_data_id);pk"`
	BaseFromNationalStatisticsIndexId int
	IndexCode                         string
	DataTime                          string
	Value                             string
	CreateTime                        time.Time
	ModifyTime                        time.Time
}

// AfterFind 在该模型上设置钩子函数，把日期转成正确的string，所以查询函数只能用Find函数，First或者Scan是不会触发该函数的来获取数据
func (m *BaseFromNationalStatisticsData) AfterFind(db *gorm.DB) (err error) {
	m.DataTime = utils.GormDateStrToDateStr(m.DataTime)

	return
}

// GetBaseFromNationalStatisticsDataByCondition 获取指标值
func GetBaseFromNationalStatisticsDataByCondition(condition string, pars []interface{}) (list []*BaseFromNationalStatisticsData, err error) {
	//o := orm.NewOrm()
	sql := `SELECT * FROM base_from_national_statistics_data WHERE 1=1 `
	if condition != "" {
		sql += condition
	}
	//_, err = o.Raw(sql, pars).QueryRows(&list)
	err = global.DEFAULT_DB.Raw(sql, pars...).Find(&list).Error
	return
}

// AddEdbDataFromNationalStatistics 新增国家统计局指标数据
func AddEdbDataFromNationalStatistics(edbCode string) (err error) {
	//o := orm.NewOrm()

	var condition string
	var pars []interface{}
	if edbCode != "" {
		condition += ` AND index_code = ? `
		pars = append(pars, edbCode)
	}

	dataList, err := GetBaseFromNationalStatisticsDataByCondition(condition, pars)
	if err != nil && !utils.IsErrNoRow(err) {
		return
	}
	var isAdd bool
	addSql := `INSERT INTO edb_data_national_statistics(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	addSql = utils.ReplaceDriverKeywords("", addSql)
	existMap := make(map[string]string)
	for _, sv := range dataList {
		eDate := sv.DataTime
		dataTime, err := time.Parse(utils.FormatDate, eDate)
		if err != nil {
			return err
		}
		timestamp := dataTime.UnixNano() / 1e6
		timeStr := fmt.Sprintf("%d", timestamp)
		if _, ok := existMap[eDate]; !ok {
			addSql += GetAddSql("0", edbCode, eDate, timeStr, sv.Value)
			isAdd = true
		}
		existMap[eDate] = sv.Value
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		utils.FileLog.Info("addSql:" + addSql)
		//_, err = o.Raw(addSql).Exec()
		err = global.DEFAULT_DB.Exec(addSql).Error
		if err != nil {
			return err
		}
	}
	return
}

// RefreshEdbDataFromNationalStatistics 刷新国家统计局指标数据
func RefreshEdbDataFromNationalStatistics(edbInfoId int, edbCode, startDate string) (err error) {
	source := utils.DATA_SOURCE_NATIONAL_STATISTICS
	subSource := utils.DATA_SUB_SOURCE_EDB

	//o := orm.NewOrm()
	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)

	var condition string
	var pars []interface{}
	if edbCode != "" {
		condition += ` AND index_code = ? `
		pars = append(pars, edbCode)
	}
	if startDate != "" {
		condition += ` AND data_time >= ? `
		pars = append(pars, startDate)
	}

	dataList, err := GetBaseFromNationalStatisticsDataByCondition(condition, pars)
	if err != nil {
		return
	}

	// 真实数据的最大日期  ， 插入规则配置的日期
	var realDataMaxDate, edbDataInsertConfigDate time.Time
	var edbDataInsertConfig *EdbDataInsertConfig
	var isFindConfigDateRealData bool //是否找到配置日期的实际数据的值
	{
		edbDataInsertConfig, err = GetEdbDataInsertConfigByEdbId(edbInfoId)
		if err != nil && !utils.IsErrNoRow(err) {
			return
		}
		if edbDataInsertConfig != nil {
			edbDataInsertConfigDate = edbDataInsertConfig.Date
		}
	}

	var existCondition string
	var existPars []interface{}

	existCondition += ` AND edb_info_id = ? `
	existPars = append(existPars, edbInfoId)
	if startDate != "" {
		existCondition += ` AND data_time >= ? `
		existPars = append(existPars, startDate)
	}

	existList, err := GetEdbDataByCondition(source, subSource, existCondition, existPars)
	if err != nil {
		return err
	}
	existMap := make(map[string]*EdbInfoSearchData)
	for _, v := range existList {
		existMap[v.DataTime] = v
	}
	addSql := ` INSERT INTO edb_data_national_statistics(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	var isAdd bool
	for _, v := range dataList {
		item := v
		eDate := item.DataTime
		dataTime, err := time.ParseInLocation(utils.FormatDate, eDate, time.Local)
		if err != nil {
			return err
		}
		if findItem, ok := existMap[v.DataTime]; !ok {
			sValue := item.Value

			timestamp := dataTime.UnixNano() / 1e6
			timeStr := fmt.Sprintf("%d", timestamp)

			addSql += GetAddSql(edbInfoIdStr, edbCode, eDate, timeStr, sValue)
			isAdd = true
		} else {
			if findItem != nil && utils.SubFloatToString(findItem.Value, 30) != item.Value {
				err = ModifyEdbDataById(source, subSource, findItem.EdbDataId, item.Value)
				if err != nil {
					return err
				}
			}
		}

		// 下面代码主要目的是处理掉手动插入的数据判断
		{
			if realDataMaxDate.IsZero() || dataTime.After(realDataMaxDate) {
				realDataMaxDate = dataTime
			}
			if edbDataInsertConfigDate.IsZero() || dataTime.Equal(edbDataInsertConfigDate) {
				isFindConfigDateRealData = true
			}
		}
	}

	// 处理手工数据补充的配置
	HandleConfigInsertEdbData(realDataMaxDate, edbDataInsertConfig, edbInfoId, source, subSource, existMap, isFindConfigDateRealData)

	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		//_, err = o.Raw(addSql).Exec()
		err = global.DEFAULT_DB.Exec(addSql).Error
		if err != nil {
			fmt.Println("RefreshEdbDataFromNationalStatistics add Err", err.Error())
			return
		}
	}
	return
}
