package models

import (
	"eta/eta_index_lib/global"
	"eta/eta_index_lib/utils"
	"fmt"
	"gorm.io/gorm"
	"strconv"
	"strings"
	"time"
)

type BaseFromOilchemIndex struct {
	BaseFromOilchemIndexId int       // 主键ID
	IndexCode              string    // 指标编码
	IndexName              string    // 指标名称
	ClassifyId             uint      // 分类ID
	Unit                   string    // 单位
	Frequency              string    // 频度
	Describe               string    // 指标描述
	Sort                   int       // 排序
	CreateTime             time.Time // 创建时间
	ModifyTime             time.Time // 修改时间
}

type BaseFromOilchemData struct {
	BaseFromOilchemDataId  int    // 数据表ID
	BaseFromOilchemIndexId int    // 指标ID
	IndexCode              string // 指标编码
	DataTime               string
	Value                  string
	CreateTime             time.Time // 创建时间
	ModifyTime             time.Time // 修改时间
}

func (e *BaseFromOilchemData) AfterFind(db *gorm.DB) (err error) {
	e.DataTime = utils.GormDateStrToDateStr(e.DataTime)

	return
}

// 添加数据
func AddBaseFromOilchemIndexMuti(items []*BaseFromOilchemIndex) (err error) {
	//o := orm.NewOrm()
	//_, err = o.InsertMulti(500, items)
	err = global.DEFAULT_DB.CreateInBatches(items, utils.MultiAddNum).Error
	return
}
func AddBaseFromOilchemIndex(item *BaseFromOilchemIndex) (lastId int64, err error) {
	//o := orm.NewOrm()
	//lastId, err = o.Insert(item)
	err = global.DEFAULT_DB.Create(&item).Error
	if err != nil {
		return
	}
	lastId = int64(item.BaseFromOilchemIndexId)
	return
}

func AddBaseFromOilchemData(item *BaseFromOilchemData) (lastId int64, err error) {
	//o := orm.NewOrm()
	//lastId, err = o.Insert(item)
	err = global.DEFAULT_DB.Create(&item).Error
	if err != nil {
		return
	}
	lastId = int64(item.BaseFromOilchemDataId)
	return
}

func AddEdbDataFromOilchem(edbCode string) (err error) {
	//o := orm.NewOrm()

	oilchemBaseDataAll, err := GetOilchemDataByCode(edbCode)
	if err != nil && !utils.IsErrNoRow(err) {
		return
	}

	var isAdd bool
	addSql := ` INSERT INTO edb_data_oilchem(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	existMap := make(map[string]string)
	for _, sv := range oilchemBaseDataAll {
		eDate := sv.DataTime
		var timeStr string
		var dataTime time.Time
		var sDataTime string
		var timestamp int64

		sDataTime = eDate
		dataTime, err = time.ParseInLocation(utils.FormatDate, eDate, time.Local)
		if err != nil {
			fmt.Println("time.Parse Err:" + eDate)
			return err
		}
		timestamp = dataTime.UnixNano() / 1e6
		timeStr = fmt.Sprintf("%d", timestamp)

		value := strings.Replace(sv.Value, "%", "", -1)
		if _, ok := existMap[sDataTime]; !ok {
			addSql += GetAddSql("0", edbCode, sDataTime, timeStr, value)
			fmt.Println("edbCode：", edbCode)
			fmt.Println("sDataTime：", sDataTime)
			fmt.Println("timeStr：", timeStr)
			fmt.Println("value：", value)
			isAdd = true
		}
		existMap[eDate] = value
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		utils.FileLog.Info("addSql:" + addSql)
		//_, err = o.Raw(addSql).Exec()
		err = global.DEFAULT_DB.Exec(addSql).Error
		if err != nil {
			return err
		}
	}
	return
}

// GetOilchemDataByCode
func GetOilchemDataByCode(indexCode string) (items []*BaseFromOilchemData, err error) {
	//o := orm.NewOrm()
	sql := "SELECT * FROM base_from_oilchem_data WHERE index_code=? "
	err = global.DEFAULT_DB.Raw(sql, indexCode).Find(&items).Error
	//_, err = o.Raw(sql, indexCode).QueryRows(&items)
	return
}

// RefreshEdbDataFromOilchem 刷新隆众资讯
func RefreshEdbDataFromOilchem(edbInfoId int, edbCode, startDate string) (err error) {
	source := utils.DATA_SOURCE_OILCHEM
	subSource := utils.DATA_SUB_SOURCE_EDB

	//o := orm.NewOrm()
	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据
	var condition string
	var pars []interface{}

	if edbCode != "" {
		condition += " AND index_code=? "
		pars = append(pars, edbCode)
	}

	if startDate != "" {
		condition += " AND data_time>=? "
		pars = append(pars, startDate)
	}

	oilchemDataList, err := GetBaseFromOilchemDataByCondition(condition, pars)
	if err != nil {
		return
	}

	// 真实数据的最大日期  ， 插入规则配置的日期
	var realDataMaxDate, edbDataInsertConfigDate time.Time
	var edbDataInsertConfig *EdbDataInsertConfig
	var isFindConfigDateRealData bool //是否找到配置日期的实际数据的值
	{
		edbDataInsertConfig, err = GetEdbDataInsertConfigByEdbId(edbInfoId)
		if err != nil && !utils.IsErrNoRow(err) {
			return
		}
		if edbDataInsertConfig != nil {
			edbDataInsertConfigDate = edbDataInsertConfig.Date
		}
	}

	var existCondition string
	var existPars []interface{}

	existCondition += " AND edb_info_id=? "
	existPars = append(existPars, edbInfoId)
	if startDate != "" {
		existCondition += " AND data_time>=? "
		existPars = append(existPars, startDate)
	}

	existList, err := GetEdbDataByCondition(source, subSource, existCondition, existPars)
	if err != nil {
		return err
	}
	existMap := make(map[string]*EdbInfoSearchData)
	for _, v := range existList {
		existMap[v.DataTime] = v
	}
	addSql := ` INSERT INTO edb_data_oilchem(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	var isAdd bool
	for _, v := range oilchemDataList {
		item := v

		eDate := item.DataTime
		dataTime, err := time.ParseInLocation(utils.FormatDate, eDate, time.Local)
		if err != nil {
			return err
		}
		if findItem, ok := existMap[v.DataTime]; !ok {
			sValue := item.Value
			timestamp := dataTime.UnixNano() / 1e6
			timeStr := fmt.Sprintf("%d", timestamp)

			addSql += GetAddSql(edbInfoIdStr, edbCode, eDate, timeStr, sValue)
			isAdd = true
		} else {
			if findItem != nil && utils.SubFloatToString(findItem.Value, 30) != item.Value {
				err = ModifyEdbDataById(source, subSource, findItem.EdbDataId, item.Value)
				if err != nil {
					return err
				}
			}
		}

		// 下面代码主要目的是处理掉手动插入的数据判断
		{
			if realDataMaxDate.IsZero() || dataTime.After(realDataMaxDate) {
				realDataMaxDate = dataTime
			}
			if edbDataInsertConfigDate.IsZero() || dataTime.Equal(edbDataInsertConfigDate) {
				isFindConfigDateRealData = true
			}
		}
	}

	// 处理手工数据补充的配置
	HandleConfigInsertEdbData(realDataMaxDate, edbDataInsertConfig, edbInfoId, source, subSource, existMap, isFindConfigDateRealData)

	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		//_, err = o.Raw(addSql).Exec()
		err = global.DEFAULT_DB.Exec(addSql).Error
		if err != nil {
			fmt.Println("RefreshEdbDataFromSci add Err", err.Error())
			return
		}
	}
	return
}

func GetBaseFromOilchemDataByCondition(condition string, pars []interface{}) (list []*BaseFromOilchemData, err error) {
	//o := orm.NewOrm()
	sql := `SELECT * FROM base_from_oilchem_data WHERE 1=1 `
	if condition != "" {
		sql += condition
	}
	//_, err = o.Raw(sql, pars).QueryRows(&list)
	err = global.DEFAULT_DB.Raw(sql, pars...).Find(&list).Error
	return
}

type BaseFromOilchemIndexReq struct {
	BaseFromOilchemIndexId int       // 主键ID
	IndexCode              string    // 指标编码
	IndexName              string    // 指标名称
	ClassifyId             uint      // 分类ID
	Unit                   string    // 单位
	Source                 string    // 数据来源
	Frequency              string    // 频度
	Describe               string    // 指标描述
	DataTime               string    // 数据日期
	Value                  string    // 数据值
	Sort                   int       // 排序
	CreateTime             time.Time // 创建时间
	ModifyTime             time.Time // 修改时间
	IndexNameStr           string    // 指标名称字符串
	MarketName             string    // 市场名称
}

type HandleOielchemEdbDataReq struct {
	List []*BaseFromOilchemIndexReq
}

func GetBaseFromOilchemIndex() (list []*BaseFromOilchemIndex, err error) {
	//o := orm.NewOrm()
	sql := `SELECT * FROM base_from_oilchem_index group by index_name `
	//_, err = o.Raw(sql).QueryRows(&list)
	err = global.DEFAULT_DB.Raw(sql).Find(&list).Error
	return
}

func GetBaseFromOilchemData(indexCode, dataTime string) (item *BaseFromOilchemData, err error) {
	//o := orm.NewOrm()
	sql := `SELECT * FROM base_from_oilchem_data where index_code=? And data_time = ? `
	//err = o.Raw(sql, indexCode, dataTime).QueryRow(&item)
	err = global.DEFAULT_DB.Raw(sql, indexCode, dataTime).First(&item).Error
	return
}

// UpdateBaseFromSci99Data
func UpdateBaseFromOilchemData(value, indexCode, dataTime string) (err error) {
	//o := orm.NewOrm()
	sql := `UPDATE base_from_sci99_data SET value=?,modify_time=NOW() WHERE index_code = ? AND data_time = ? `
	//_, err = o.Raw(sql, value, indexCode, dataTime).Exec()
	err = global.DEFAULT_DB.Exec(sql, value, indexCode, dataTime).Error
	return
}
