package models

import (
	"eta/eta_index_lib/global"
	"eta/eta_index_lib/utils"
	"gorm.io/gorm"
	"time"
)

type ChartEdbMapping struct {
	ChartEdbMappingId int       `gorm:"column:chart_edb_mapping_id;type:int(11);primaryKey;not null;" `
	ChartInfoId       int       `gorm:"column:chart_info_id;type:int(11);comment:图表id;default:0;"`                  // 图表id
	EdbInfoId         int       `gorm:"column:edb_info_id;type:int(11);comment:指标id;default:0;"`                    // 指标id
	CreateTime        time.Time `gorm:"column:create_time;type:datetime;comment:创建时间;default:NULL;"`                // 创建时间
	ModifyTime        time.Time `gorm:"column:modify_time;type:datetime;comment:修改时间;default:NULL;"`                // 修改时间
	UniqueCode        string    `gorm:"column:unique_code;type:varchar(50);comment:唯一编码;"`                          // 唯一编码
	MaxData           float64   `gorm:"column:max_data;type:double;comment:上限;default:0;"`                          // 上限
	MinData           float64   `gorm:"column:min_data;type:double;comment:下限;default:0;"`                          // 下限
	IsOrder           bool      `gorm:"column:is_order;type:tinyint(4);comment:1:正序,0:逆序;default:1;"`               // 1:正序,0:逆序
	IsAxis            int       `gorm:"column:is_axis;type:tinyint(4);comment:1:左轴,0:右轴;default:1;"`                // 1:左轴,0:右轴
	EdbInfoType       int       `gorm:"column:edb_info_type;type:tinyint(4);comment:1:标准指标,0:领先指标;default:1;"`      // 1:标准指标,0:领先指标
	LeadValue         int       `gorm:"column:lead_value;type:int(11);comment:领先值;default:0;"`                      // 领先值
	LeadUnit          string    `gorm:"column:lead_unit;type:varchar(10);comment:领先单位;"`                            // 领先单位
	ChartStyle        string    `gorm:"column:chart_style;type:varchar(20);comment:图表类型;"`                          // 图表类型
	ChartColor        string    `gorm:"column:chart_color;type:varchar(36);comment:颜色;"`                            // 颜色
	PredictChartColor string    `gorm:"column:predict_chart_color;type:varchar(36);comment:预测数据的颜色;"`               // 预测数据的颜色
	ChartWidth        float64   `gorm:"column:chart_width;type:double;comment:线条大小;default:0;"`                     // 线条大小
	Source            int       `gorm:"column:source;type:tinyint(9) UNSIGNED;comment:1：ETA图库；2：商品价格曲线;default:1;"` // 1：ETA图库；2：商品价格曲线
	EdbAliasName      string    `gorm:"column:edb_alias_name;type:varchar(255);comment:中文别名;"`                      // 中文别名
	IsConvert         int       `gorm:"column:is_convert;type:int(11);comment:是否数据转换 0不转 1转;default:0;" `           // 是否数据转换 0不转 1转
	ConvertType       int       `gorm:"column:convert_type;type:int(11);comment:数据转换类型 1乘 2除 3对数;default:0;"`       // 数据转换类型 1乘 2除 3对数
	ConvertValue      float64   `gorm:"column:convert_value;type:double;comment:数据转换值;default:0;"`                  // 数据转换值
	ConvertUnit       string    `gorm:"column:convert_unit;type:varchar(10);comment:数据转换单位;"`                       // 数据转换单位
	ConvertEnUnit     string    `gorm:"column:convert_en_unit;type:varchar(10);comment:数据转换单位;"`                    // 数据转换单位
	ChartScale        float64   `gorm:"column:chart_scale;type:double;comment:参考刻度线;default:0;"`                    // 参考刻度线
}

// GetChartEdbMappingListByEdbInfoId 根据指标id获取关联图表列表
func GetChartEdbMappingListByEdbInfoId(edbInfoId int) (list []*ChartEdbMapping, err error) {
	sql := ` SELECT *
             FROM chart_edb_mapping AS a
			 WHERE edb_info_id=? 
             ORDER BY chart_edb_mapping_id ASC `
	err = global.DEFAULT_DB.Raw(sql, edbInfoId).Find(&list).Error

	return
}

// GetGroupChartEdbMappingListByEdbInfoId 根据指标id和来源分组获取关联图表列表
func GetGroupChartEdbMappingListByEdbInfoId(edbInfoId, source int) (list []*ChartEdbMapping, err error) {
	sql := ` SELECT *
             FROM chart_edb_mapping AS a
			 WHERE edb_info_id=? AND source = ?  GROUP BY chart_info_id `
	err = global.DEFAULT_DB.Raw(sql, edbInfoId, source).Find(&list).Error

	return
}

type ChartEdbInfoMapping struct {
	EdbInfoId           int     `description:"指标id"`
	SourceName          string  `description:"来源名称"`
	Source              int     `description:"来源id"`
	SubSource           int     `description:"来源id"`
	EdbCode             string  `description:"指标编码"`
	EdbName             string  `description:"指标名称"`
	EdbAliasName        string  `description:"指标名称（别名）"`
	EdbNameEn           string  `description:"英文指标名称"`
	EdbAliasNameEn      string  `description:"英文指标名称（别名）"`
	EdbType             int     `description:"指标类型:1:基础指标,2:计算指标"`
	Frequency           string  `description:"频率"`
	FrequencyEn         string  `description:"英文频率"`
	Unit                string  `description:"单位"`
	UnitEn              string  `description:"英文单位"`
	StartDate           string  `description:"起始日期"`
	EndDate             string  `description:"终止日期"`
	ModifyTime          string  `description:"指标最后更新时间"`
	ChartEdbMappingId   int     `description:"图表指标id"`
	ChartInfoId         int     `description:"图表id"`
	MaxData             float64 `description:"上限"`
	MinData             float64 `description:"下限"`
	IsOrder             bool    `description:"true:正序,false:逆序"`
	IsAxis              int     `description:"1:左轴,0:右轴"`
	EdbInfoType         int     `description:"1:标准指标,0:领先指标"`
	EdbInfoCategoryType int     `description:"0:普通指标,1:预测指标"`
	LeadValue           int     `description:"领先值"`
	LeadUnit            string  `description:"领先单位"`
	LeadUnitEn          string  `description:"领先英文单位"`
	ChartStyle          string  `description:"图表类型"`
	ChartColor          string  `description:"颜色"`
	PredictChartColor   string  `description:"预测数据的颜色"`
	ChartWidth          float64 `description:"线条大小"`
	ChartType           int     `description:"生成样式:1:曲线图,2:季节性图,3:面积图，4：柱状图，5：散点图，6：组合图，7：柱方图，8：商品价格曲线图，9：相关性图"`
	LatestDate          string  `description:"数据最新日期"`
	LatestValue         float64 `description:"数据最新值"`
	MoveLatestDate      string  `description:"移动后的数据最新日期"`
	UniqueCode          string  `description:"指标唯一编码"`
	MinValue            float64 `json:"-" description:"最小值"`
	MaxValue            float64 `json:"-" description:"最大值"`
	DataList            interface{}
	IsNullData          bool    `json:"-" description:"是否空数据"`
	MappingSource       int     `description:"1：ETA图库；2：商品价格曲线"`
	RegionType          string  `description:"交易所来源，海外还是国内" json:"-"`
	ClassifyId          int     `description:"分类id"`
	SubSourceName       string  `description:"子数据来源名称"`
	IndicatorCode       string  `description:"指标代码"`
	IsConvert           int     `description:"是否数据转换 0不转 1转"`
	ConvertType         int     `description:"数据转换类型 1乘 2除 3对数"`
	ConvertValue        float64 `description:"数据转换值"`
	ConvertUnit         string  `description:"数据转换单位"`
	ConvertEnUnit       string  `description:"数据转换单位"`
	IsJoinPermission    int     `description:"是否加入权限管控，0：不加入；1：加入；默认：0"`
	HaveOperaAuth       bool    `description:"是否有数据权限，默认：false"`
	UniqueFlag          string  `description:"唯一标识(与唯一编码不是一个东西)"`
}

// AfterFind 在该模型上设置钩子函数，把日期转成正确的string，所以查询函数只能用Find函数，First或者Scan是不会触发该函数的来获取数据
func (m *ChartEdbInfoMapping) AfterFind(db *gorm.DB) (err error) {
	m.StartDate = utils.GormDateStrToDateStr(m.StartDate)
	m.EndDate = utils.GormDateStrToDateStr(m.EndDate)
	m.LatestDate = utils.GormDateStrToDateStr(m.LatestDate)

	return
}

// GetChartEdbMappingByEdbInfoId 根据指标id获取edb_mapping
func GetChartEdbMappingByEdbInfoId(edbInfoId int) (item *ChartEdbInfoMapping, err error) {
	sql := ` SELECT edb_info_id,source_name,classify_id,source,sub_source,edb_code,edb_name,edb_name_en,frequency,unit,unit_en,start_date,end_date,modify_time,latest_date,latest_value,unique_code,edb_info_type AS edb_info_category_type,edb_type,max_value,min_value,is_join_permission
             FROM edb_info
			 WHERE edb_info_id = ? limit 1`
	err = global.DEFAULT_DB.Raw(sql, edbInfoId).First(&item).Error

	return
}
