package excel

import (
	"eta/eta_index_lib/global"
	"time"
)

// ExcelClassify excel表格分类
type ExcelClassify struct {
	ExcelClassifyId   int       `gorm:"primaryKey;autoIncrement;column:excel_classify_id" description:"表格分类ID"`
	Source            int       `gorm:"column:source" description:"表格来源，1：excel插件的表格，2：自定义表格，3：混合表格，4：自定义分析，默认：1"`
	ExcelClassifyName string    `gorm:"column:excel_classify_name" description:"分类名称"`
	ParentId          int       `gorm:"column:parent_id" description:"父级id"`
	SysUserId         int       `gorm:"column:sys_user_id" description:"创建人id"`
	SysUserRealName   string    `gorm:"column:sys_user_real_name" description:"创建人姓名"`
	Level             int       `gorm:"column:level" description:"层级"`
	UniqueCode        string    `gorm:"column:unique_code" description:"唯一编码"`
	Sort              int       `gorm:"column:sort" description:"排序字段，越小越靠前，默认值：10"`
	IsDelete          int       `gorm:"column:is_delete" description:"是否删除，0：未删除，1：已删除（注意：这里的描述可能与原字段名IsDelete的注释不完全一致，已根据常见用法调整）"`
	CreateTime        time.Time `gorm:"column:create_time" description:"创建时间"`
	ModifyTime        time.Time `gorm:"column:modify_time" description:"修改时间"`
}

// AddExcelClassify 添加excel分类
func AddExcelClassify(item *ExcelClassify) (lastId int64, err error) {
	err = global.DEFAULT_DB.Create(item).Error
	if err != nil {
		return
	}
	lastId = int64(item.ExcelClassifyId)

	return
}

type ExcelClassifyItems struct {
	ExcelClassifyId   int `description:"分类id"`
	ExcelInfoId       int `description:"表格id"`
	ExcelClassifyName string
	ParentId          int
	Level             int    `description:"层级"`
	Sort              int    `description:"排序字段，越小越靠前，默认值：10"`
	UniqueCode        string `description:"唯一编码"`
	SysUserId         int    `description:"创建人id"`
	SysUserRealName   string `description:"创建人姓名"`
	StartDate         string `description:"自定义开始日期"`
	Children          []*ExcelClassifyItems
}

// GetExcelClassifyByParentId
// @Description: 根据父级分类id获取指标下面的分类列表
// @author: Roc
// @datetime2023-10-30 13:46:35
// @param parentId int
// @param source int
// @return items []*ExcelClassifyItems
// @return err error
func GetExcelClassifyByParentId(parentId, source int) (items []*ExcelClassifyItems, err error) {
	sql := ` SELECT * FROM excel_classify WHERE parent_id=? AND source = ? AND is_delete=0 order by sort asc,excel_classify_id asc`
	err = global.DEFAULT_DB.Raw(sql, parentId, source).Find(&items).Error

	return
}
